@@ -19,19 +19,17 @@
  */
 
 #include "config.h"
+#include "ephy-notification.h"
 #include "ephy-notification-container.h"
 
 struct _EphyNotificationContainer {
-  GtkRevealer parent_instance;
+  AdwBin parent_instance;
 
-  GtkWidget *grid;
+  GtkWidget *revealer;
+  GtkWidget *box;
 };
 
-struct _EphyNotificationContainerClass {
-  GtkRevealerClass parent_class;
-};
-
-G_DEFINE_TYPE (EphyNotificationContainer, ephy_notification_container, GTK_TYPE_REVEALER);
+G_DEFINE_FINAL_TYPE (EphyNotificationContainer, ephy_notification_container, ADW_TYPE_BIN);
 
 static EphyNotificationContainer *notification_container = NULL;
 
@@ -47,10 +45,13 @@ ephy_notification_container_init (EphyNotificationContainer *self)
   gtk_widget_set_halign (GTK_WIDGET (self), GTK_ALIGN_CENTER);
   gtk_widget_set_valign (GTK_WIDGET (self), GTK_ALIGN_START);
 
-  self->grid = gtk_grid_new ();
-  gtk_orientable_set_orientation (GTK_ORIENTABLE (self->grid), GTK_ORIENTATION_VERTICAL);
-  gtk_grid_set_row_spacing (GTK_GRID (self->grid), 6);
-  gtk_container_add (GTK_CONTAINER (self), self->grid);
+  self->revealer = gtk_revealer_new ();
+  adw_bin_set_child (ADW_BIN (self), self->revealer);
+
+  self->box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
+  gtk_revealer_set_child (GTK_REVEALER (self->revealer), self->box);
+
+  gtk_widget_set_visible (GTK_WIDGET (self), FALSE);
 }
 
 static void
@@ -68,29 +69,41 @@ ephy_notification_container_get_default (void)
                        NULL);
 }
 
+static void
+notification_close_cb (EphyNotification          *notification,
+                       EphyNotificationContainer *self)
+{
+  gtk_box_remove (GTK_BOX (self->box), GTK_WIDGET (notification));
+
+  if (!gtk_widget_get_first_child (self->box)) {
+    gtk_widget_set_visible (GTK_WIDGET (self), FALSE);
+    gtk_revealer_set_reveal_child (GTK_REVEALER (self->revealer), FALSE);
+  }
+}
+
 void
 ephy_notification_container_add_notification (EphyNotificationContainer *self,
                                               GtkWidget                 *notification)
 {
+  GtkWidget *child;
+
   g_assert (EPHY_IS_NOTIFICATION_CONTAINER (self));
   g_assert (GTK_IS_WIDGET (notification));
 
-  gtk_container_add (GTK_CONTAINER (self->grid), notification);
-  gtk_widget_show_all (GTK_WIDGET (self));
-  gtk_revealer_set_reveal_child (GTK_REVEALER (self), TRUE);
-}
+  for (child = gtk_widget_get_first_child (self->box);
+       child;
+       child = gtk_widget_get_next_sibling (child)) {
+    EphyNotification *child_notification = EPHY_NOTIFICATION (child);
 
-guint
-ephy_notification_container_get_num_children (EphyNotificationContainer *self)
-{
-  GList *children;
-  guint retval;
+    if (ephy_notification_is_duplicate (child_notification, EPHY_NOTIFICATION (notification))) {
+      gtk_box_remove (GTK_BOX (self->box), notification);
+      return;
+    }
+  }
 
-  g_assert (EPHY_IS_NOTIFICATION_CONTAINER (self));
+  gtk_box_append (GTK_BOX (self->box), notification);
+  gtk_widget_set_visible (GTK_WIDGET (self), TRUE);
+  gtk_revealer_set_reveal_child (GTK_REVEALER (self->revealer), TRUE);
 
-  children = gtk_container_get_children (GTK_CONTAINER (self->grid));
-  retval = g_list_length (children);
-  g_list_free (children);
-
-  return retval;
+  g_signal_connect (notification, "close", G_CALLBACK (notification_close_cb), self);
 }
