@@ -29,7 +29,7 @@
 #include <string.h>
 #include <sys/stat.h>
 #include <unistd.h>
-#include <webkit2/webkit2.h>
+#include <webkit/webkit.h>
 
 struct _EphySnapshotService {
   GObject parent_instance;
@@ -37,7 +37,7 @@ struct _EphySnapshotService {
   GHashTable *cache;
 };
 
-G_DEFINE_TYPE (EphySnapshotService, ephy_snapshot_service, G_TYPE_OBJECT)
+G_DEFINE_FINAL_TYPE (EphySnapshotService, ephy_snapshot_service, G_TYPE_OBJECT)
 
 typedef enum {
   SNAPSHOT_STALE,
@@ -212,19 +212,21 @@ out:
 }
 
 static GdkPixbuf *
-ephy_snapshot_service_prepare_snapshot (cairo_surface_t *surface)
+ephy_snapshot_service_prepare_snapshot (GdkTexture *texture)
 {
   GdkPixbuf *snapshot, *scaled;
   int orig_width, orig_height;
 
-  orig_width = cairo_image_surface_get_width (surface);
-  orig_height = cairo_image_surface_get_height (surface);
+  orig_width = gdk_texture_get_width (texture);
+  orig_height = gdk_texture_get_height (texture);
+
+  if (!orig_width || !orig_height)
+    return NULL;
+
+  snapshot = gdk_pixbuf_get_from_texture (texture);
 
   if (orig_width < EPHY_THUMBNAIL_WIDTH ||
       orig_height < EPHY_THUMBNAIL_HEIGHT) {
-    snapshot = gdk_pixbuf_get_from_surface (surface,
-                                            0, 0,
-                                            orig_width, orig_height);
     scaled = gdk_pixbuf_scale_simple (snapshot,
                                       EPHY_THUMBNAIL_WIDTH,
                                       EPHY_THUMBNAIL_HEIGHT,
@@ -233,7 +235,6 @@ ephy_snapshot_service_prepare_snapshot (cairo_surface_t *surface)
     gfloat width_ratio = (gfloat)EPHY_THUMBNAIL_WIDTH / (gfloat)orig_width;
     gfloat new_height = orig_height * width_ratio;
 
-    snapshot = gdk_pixbuf_get_from_surface (surface, 0, 0, orig_width, orig_height);
     scaled = gdk_pixbuf_scale_simple (snapshot,
                                       EPHY_THUMBNAIL_WIDTH,
                                       new_height,
@@ -301,15 +302,13 @@ typedef struct {
   SnapshotPathCachedData *data;
 } CacheData;
 
-static gboolean
+static void
 idle_cache_snapshot_path (gpointer user_data)
 {
   CacheData *data = (CacheData *)user_data;
   g_hash_table_insert (data->cache, data->url, data->data);
   g_hash_table_unref (data->cache);
   g_free (data);
-
-  return G_SOURCE_REMOVE;
 }
 
 static void
@@ -325,7 +324,7 @@ cache_snapshot_data_in_idle (EphySnapshotService   *service,
   data->data = g_new (SnapshotPathCachedData, 1);
   data->data->path = g_strdup (path);
   data->data->freshness = freshness;
-  g_idle_add (idle_cache_snapshot_path, data);
+  g_idle_add_once (idle_cache_snapshot_path, data);
 }
 
 static void
@@ -390,12 +389,20 @@ snapshot_saved (EphySnapshotService *service,
 }
 
 static void
-save_snapshot (cairo_surface_t *surface,
-               GTask           *task)
+save_snapshot (GdkTexture *texture,
+               GTask      *task)
 {
   SnapshotAsyncData *data = g_task_get_task_data (task);
 
-  data->snapshot = ephy_snapshot_service_prepare_snapshot (surface);
+  data->snapshot = ephy_snapshot_service_prepare_snapshot (texture);
+  if (!data->snapshot) {
+    g_task_return_new_error (task,
+                             EPHY_SNAPSHOT_SERVICE_ERROR,
+                             EPHY_SNAPSHOT_SERVICE_ERROR_WEB_VIEW,
+                             "WebView returned invalid snapshot for \"%s\"", data->url);
+    g_object_unref (task);
+    return;
+  }
 
   ephy_snapshot_service_save_snapshot_async (g_task_get_source_object (task),
                                              data->snapshot,
@@ -410,18 +417,17 @@ on_snapshot_ready (WebKitWebView *web_view,
                    GAsyncResult  *result,
                    GTask         *task)
 {
-  cairo_surface_t *surface;
+  g_autoptr (GdkTexture) texture = NULL;
   GError *error = NULL;
 
-  surface = webkit_web_view_get_snapshot_finish (web_view, result, &error);
+  texture = webkit_web_view_get_snapshot_finish (web_view, result, &error);
   if (error) {
     g_task_return_error (task, error);
     g_object_unref (task);
     return;
   }
 
-  save_snapshot (surface, task);
-  cairo_surface_destroy (surface);
+  save_snapshot (texture, task);
 }
 
 static gboolean
@@ -644,6 +650,8 @@ take_fresh_snapshot_in_background_if_stale (EphySnapshotService *service,
                           data,
                           (GDestroyNotify)snapshot_async_data_free);
     ephy_snapshot_service_take_from_webview (task);
+  } else {
+    snapshot_async_data_free (data);
   }
 }
 
@@ -662,7 +670,6 @@ got_snapshot_path_for_url (EphySnapshotService *service,
                            GAsyncResult        *result,
                            GTask               *task)
 {
-#ifndef __clang_analyzer__
   SnapshotAsyncData *data = g_task_get_task_data (task);
   char *path;
 
@@ -674,7 +681,6 @@ got_snapshot_path_for_url (EphySnapshotService *service,
   } else {
     ephy_snapshot_service_take_from_webview (task);
   }
-#endif
 }
 
 void
