@@ -19,7 +19,7 @@
 
 #pragma once
 
-#include <dazzle.h>
+#include "dzl-suggestion.h"
 
 G_BEGIN_DECLS
 
@@ -42,5 +42,7 @@ const char     *ephy_suggestion_get_uri                  (EphySuggestion *self);
 
 void            ephy_suggestion_set_favicon              (EphySuggestion  *self,
                                                           cairo_surface_t *favicon);
+void            ephy_suggestion_set_secondary_icon       (EphySuggestion  *self,
+                                                          const char      *icon_name);
 
 G_END_DECLS
