@@ -45,7 +45,7 @@ struct _EphyBookmarksPopover {
   EphyBookmarksManager *manager;
 };
 
-G_DEFINE_TYPE (EphyBookmarksPopover, ephy_bookmarks_popover, GTK_TYPE_POPOVER)
+G_DEFINE_FINAL_TYPE (EphyBookmarksPopover, ephy_bookmarks_popover, GTK_TYPE_POPOVER)
 
 #define EPHY_LIST_BOX_ROW_TYPE_BOOKMARK "bookmark"
 #define EPHY_LIST_BOX_ROW_TYPE_TAG "tag"
@@ -319,17 +319,21 @@ ephy_bookmarks_popover_tag_created_cb (EphyBookmarksPopover *self,
 static void
 ephy_bookmarks_popover_tag_deleted_cb (EphyBookmarksPopover *self,
                                        const char           *tag,
-                                       int                   position,
                                        EphyBookmarksManager *manager)
 {
   GtkListBoxRow *row;
+  int i = 0;
 
   g_assert (EPHY_IS_BOOKMARKS_POPOVER (self));
   g_assert (EPHY_IS_BOOKMARKS_MANAGER (manager));
 
-  row = gtk_list_box_get_row_at_index (GTK_LIST_BOX (self->tags_list_box), position);
-  gtk_list_box_remove (GTK_LIST_BOX (self->tags_list_box),
-                       GTK_WIDGET (row));
+  while ((row = gtk_list_box_get_row_at_index (GTK_LIST_BOX (self->tags_list_box), i++))) {
+    const char *title = g_object_get_data (G_OBJECT (row), "title");
+    if (g_strcmp0 (title, tag) == 0) {
+      gtk_list_box_remove (GTK_LIST_BOX (self->tags_list_box), GTK_WIDGET (row));
+      break;
+    }
+  }
 
   if (g_strcmp0 (gtk_stack_get_visible_child_name (GTK_STACK (self->toplevel_stack)), "tag_detail") == 0 &&
       g_strcmp0 (self->tag_detail_tag, tag) == 0) {
@@ -396,21 +400,6 @@ ephy_bookmarks_popover_show_tag_detail (EphyBookmarksPopover *self,
   g_sequence_free (bookmarks);
 }
 
-static void
-ephy_bookmarks_popover_open_bookmark (EphyBookmarksPopover *self,
-                                      GtkListBoxRow        *row,
-                                      EphyLinkFlags         flags)
-{
-  GtkWidget *window;
-  const char *url;
-
-  window = gtk_widget_get_ancestor (GTK_WIDGET (self), EPHY_TYPE_WINDOW);
-  g_assert (EPHY_IS_WINDOW (window));
-  url = ephy_bookmark_row_get_bookmark_url (EPHY_BOOKMARK_ROW (row));
-
-  ephy_link_open (EPHY_LINK (window), url, NULL, flags | EPHY_LINK_BOOKMARK);
-}
-
 static void
 row_clicked_cb (GtkGesture           *gesture,
                 int                   n_click,
@@ -448,7 +437,7 @@ row_clicked_cb (GtkGesture           *gesture,
 
     flags = ephy_link_flags_from_modifiers (modifiers, button == GDK_BUTTON_MIDDLE);
 
-    ephy_bookmarks_popover_open_bookmark (self, row, flags);
+    ephy_bookmark_row_open (EPHY_BOOKMARK_ROW (row), flags);
 
     if (button != GDK_BUTTON_MIDDLE)
       gtk_popover_popdown (GTK_POPOVER (self));
